__all__ = ["factory_dict"]


def factory_dict(factory_function, dict_type=dict, provide_key=True):
    """Produces a dictionary object with default values for missing keys

    :param factory_function: function to create value for missing key
    :param dict_type: base class for returned dictionary
    :param provide_key: provide the factory function with the missing key
    """
    def missing_key(self, key):
        value = self[key] = factory_function(key)
        return value

    def missing(self, key):
        value = self[key] = factory_function()
        return value

    callback = missing_key if provide_key else missing
    dict_cls = type("factory_dict", (dict_type,), {"__missing__": callback})
    return dict_cls()
